/*

===== IMPORTANT =====

This is sample code demonstrating API, technology or techniques in development.
Although this sample code has been reviewed for technical accuracy, it is not
final. Apple is supplying this information to help you plan for the adoption of
the technologies and programming interfaces described herein. This information
is subject to change, and software implemented based on this sample code should
be tested with final operating system software and final documentation. Newer
versions of this sample code may be provided with future seeds of the API or
technology. For information about updates to this and other developer
documentation, view the New & Updated sidebars in subsequent documentation
seeds.

=====================

File: MasterViewController.m
Abstract: 
Controls the table and navigation between master and detail views.  


Version: 1.0

Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple Inc.
("Apple") in consideration of your agreement to the following terms, and your
use, installation, modification or redistribution of this Apple software
constitutes acceptance of these terms.  If you do not agree with these terms,
please do not use, install, modify or redistribute this Apple software.

In consideration of your agreement to abide by the following terms, and subject
to these terms, Apple grants you a personal, non-exclusive license, under
Apple's copyrights in this original Apple software (the "Apple Software"), to
use, reproduce, modify and redistribute the Apple Software, with or without
modifications, in source and/or binary forms; provided that if you redistribute
the Apple Software in its entirety and without modifications, you must retain
this notice and the following text and disclaimers in all such redistributions
of the Apple Software.
Neither the name, trademarks, service marks or logos of Apple Inc. may be used
to endorse or promote products derived from the Apple Software without specific
prior written permission from Apple.  Except as expressly stated in this notice,
no other rights or licenses, express or implied, are granted by Apple herein,
including but not limited to any patent rights that may be infringed by your
derivative works or by other works in which the Apple Software may be
incorporated.

The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
COMBINATION WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR
DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF
CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF
APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (C) 2008 Apple Inc. All Rights Reserved.

*/

#import "MasterViewController.h"
// Need to import the AppDelegate because it owns the books array and we want access to that property.
#import "AppDelegate.h"
#import "DetailViewController.h"
#import "Task.h"

// Convenience macro for accessing the application delegate, cast to the appropriate class.
#define _AppDelegate_ ((AppDelegate *)[[UIApplication sharedApplication] delegate])

// Private interface for MasterViewController - internal only methods.
@interface MasterViewController (Private)
- (void)inspectBook:(Book *)book;
@end

@implementation MasterViewController

- (id)init {
    if (self = [super init]) {
        // This title is used by the built in navigation controller.
        self.title = @"ToDo Liste";
    }
    return self;
}

- (void)dealloc {
    // Release allocated resources.
    tableView.dataSource = nil;
    tableView.delegate = nil;
    [detailViewController release];
    [super dealloc];
}

// Set up the user interface.
- (void)loadView {
    // Display the standard edit button in the top left corner.
    self.navigationItem.customLeftView = self.editButton;
    // Create an add button to display in the top right corner.
    addButton = [[UIButton buttonWithType:UIButtonTypeNavigation] retain];
    [addButton setImage:[UIImage imageNamed:@"plus.png"] forStates:(UIControlStateNormal & UIControlStateHighlighted)];
    [addButton addTarget:_AppDelegate_ action:@selector(addTask) forControlEvents:UIControlEventTouchUpInside];
    self.navigationItem.customRightView = addButton;
    // Create the main table.
    tableView = [[[UITableView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame] style:UITableViewStylePlain] autorelease];
    tableView.delegate = self;
    tableView.dataSource = self;    
    self.view = tableView;
    
    // We use Key-Value-Observing to improve encapsulation between the app delegate, owner of the data, 
    // and management/updates for the views by registering to be notified when the "books" array changes.
    [_AppDelegate_ addObserver:self forKeyPath:@"books" 
            options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context:nil];
}

// Update the table before the view displays.
- (void)viewWillAppear:(BOOL)animated {
    [tableView reloadData];
}

/**
 * Wird aufgerufen wenn der Benutzer Edit ausgewaehlt hat.
 */
- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
    // Updates the appearance of the Edit|Done button as necessary.
    [super setEditing:editing animated:animated];
    [tableView setEditing:editing animated:YES];
    
	/**
	 * Im Falle von Edit wird der Button zum Hinzufuegen eines neuen Tasks ausgeblendet
	 */
    if (editing) {
        addButton.hidden = YES;
    } else {
        addButton.hidden = NO;
    }
}

// Handle change notifications for observed key paths of other objects.
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change 
        context:(void *)context {
    // In this sample, there is only one key path in which this object is interested - "books".
    // All other key path notifications will be passed along to the superclass, IFF it handles such notifications.
    if ([keyPath isEqual:@"books"]) {
        // Test the type of change. Insertions of new data receive special handling.
        if ([[change valueForKey:NSKeyValueChangeKindKey] intValue] == NSKeyValueChangeInsertion) {
            // Determine the index that changed, map that to a book object.
            NSUInteger index = [[change valueForKey:NSKeyValueChangeIndexesKey] firstIndex];
            Book *book = (Book *)[_AppDelegate_.books objectAtIndex:index];
            // Load the inspection (detail view) with the newly inserted book.
            [self inspectBook:book];
        }
    // Verify that the superclass does indeed handle these notifications before actually invoking that method.
    } else if ([[self superclass] instancesRespondToSelector:@selector(observeValueForKeyPath:ofObject:change:context:)]) {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

- (void)inspectBook:(Book *)book {
    // Create the detail view lazily
    if (detailViewController == nil) {
        detailViewController = [[DetailViewController alloc] init];
    }
    // Retrieve the other attributes of the book from the database (if needed).
    [book hydrate];
    // Set the detail controller's inspected item to the currently-selected book.
    detailViewController.book = book;
    // "Push" the detail view on to the navigation controller's stack.
    [self.navigationController pushViewController:detailViewController animated:YES];
}

#pragma mark Table Delegate and Data Source Methods
// These methods are all part of either the UITableViewDelegate or UITableViewDataSource protocols.

// This table will always only have one section.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tv {
    return 1;
}

// One row per book, the number of books is the number of rows.
- (NSInteger)tableView:(UITableView *)tv numberOfRowsInSection:(NSInteger)section {
    return [_AppDelegate_.books count];
}

// The accessory type is the image displayed on the far right of each table cell. In order for the delegate method
// tableView:accessoryButtonClickedForRowWithIndexPath: to be called, you must return the "Detail Disclosure Button" type.
- (UITableViewCellAccessoryType)tableView:(UITableView *)tv accessoryTypeForRowWithIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellAccessoryDisclosureIndicator;
}

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath 
        withAvailableCell:(UITableViewCell *)availableCell {
    UISimpleTableViewCell *cell = nil;
    if (availableCell != nil) {
        // Use the existing cell if it's there
        cell = (UISimpleTableViewCell *)availableCell;
    } else {
        // Create a new cell. CGRectZero allows the cell to determine the appropriate size.
        cell = [[[UISimpleTableViewCell alloc] initWithFrame:CGRectZero] autorelease];
    }
    // Retrieve the book object matching the row from the application delegate's array.
    Task *book = (Task *)[_AppDelegate_.books objectAtIndex:indexPath.row];
    //Als Inhalt wird die Bezeichnung des Tasks selbst genommen
	cell.text =  book.title;
	//Farbe abhaenig vom Datum setzen
	NSDate* today = [NSDate date];
	if ([today compare:book.copyright] == NSOrderedAscending) {
		cell.textColor = [UIColor yellowColor]; 
	} else if ([today compare:book.copyright] == NSOrderedDescending) {
		cell.textColor = [UIColor redColor]; 
	} else if ([today compare:book.copyright] == NSOrderedSame) {
		cell.textColor = [UIColor purpleColor]; 
	}
	
    return cell;
}

// This method is called when the user clicks on an accessory button of type "Detail Disclosure Button". The row of the cell
// of the button can be retrieved from the index path.
- (NSIndexPath *)tableView:(UITableView *)tv willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Inspect the book (method defined above).
    [self inspectBook:[_AppDelegate_.books objectAtIndex:indexPath.row]];
    return indexPath;
}

- (void)tableView:(UITableView *)tv commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
        forRowAtIndexPath:(NSIndexPath *)indexPath {
    // If row is deleted, remove it from the list.
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Find the book at the deleted row, and remove from application delegate's array.
        Task *book = [_AppDelegate_.books objectAtIndex:indexPath.row];
        [_AppDelegate_ removeBook:book];
        // Animate the deletion from the table.
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }
}

@end
